# coding: utf-8
# compute scores for examinations
# ANSWER_FILE: answer file
# RUN_FILE: RITE2 run file

import sys
import getopt

def usage():
    sys.stderr.write('Usage: python %s [-h] -g ANSWER_FILE -s SYSTEM_FILE\n' % sys.argv[0])
    sys.stderr.write('Evaluation script for RITE2 EXAM subtask\n')
    sys.stderr.write('  ANSWER_FILE: answer file provided from the organizer (*.ans)\n')
    sys.stderr.write('  SYSTEM_FILE: run file output from the system\n')
    sys.exit(1)

try:
    options, args = getopt.getopt(sys.argv[1:], 'hg:s:')
except getopt.GetoptError:
    usage()

if len(args) != 0:
    sys.stderr.write('Unknown argument: %s\n' % args[0])
    usage()

ans_name = False
run_name = False
for opt, arg in options:
    if opt == '-h':
        usage()
    elif opt == '-g':
        ans_name = arg
    elif opt == '-s':
        run_name = arg
    else:
        sys.stderr.write('Unknown option: %s\n' % opt)
        usage()

if not ans_name:
    sys.stderr.write('ANSWER_FILE not specified\n')
    usage()
if not run_name:
    sys.stderr.write('SYSTEM_FILE not specified\n')
    usage()

######################################################################
## read run file

try:
    run_file = open(run_name, 'r')
except IOError:
    sys.stderr.write('Run file error: Cannot open run file: %s\n' % run_name)
    sys.exit(1)

run_results = {}  ## outputs from the system: pair ID -> (label, confidence score)
for line in run_file:
    cols = line.split()
    if not cols: continue
    if len(cols) != 3:
        sys.stderr.write('Run file error: Wrong format for run file: %s' % line)
        sys.exit(1)
    (id, label, confidence) = cols
    if run_results.has_key(id):
        sys.stderr.write('Run file error: Multiple outputs found for ID %s: %s' % (id, line))
        sys.exit(1)
    if label != 'Y' and label != 'N':
        sys.stderr.write('Run file error: Unknown label found for ID %s: %s' % (id, line))
        sys.exit(1)
    run_results[id] = { 'label': label, 'confidence': float(confidence) }

run_file.close()

######################################################################
## read answer file and store scoring results

## find ID of correct answer
def findAnswerID(answers, label, question_id):
    answer = [id for id in answers.keys() if answers[id] == label]
    if len(answer) != 1:
        sys.stderr.write('Answer file error: answer file has multiple correct choices for Question ID %s.  Contact the organizer.\n' % question_id)
        sys.exit(1)
    return answer[0]

## find ID with highest confidence score
def chooseBestID(ids, label):
    choices = [id for id in ids if run_results[id]['label'] == label]
    if len(choices) == 0:
        ## none has the correct label -> choose ID with lowest confidence
        choices = ids
        choices.sort(lambda x, y: cmp(run_results[x]['confidence'], run_results[y]['confidence']))
    else:
        choices.sort(lambda x, y: cmp(run_results[y]['confidence'], run_results[x]['confidence']))
    return choices[0]

try:
    ans_file = open(ans_name, 'r')
except IOError:
    sys.stderr.write('Answer file error: Cannot open answer file: %s\n' % ans_name)
    sys.exit(1)

eval_results = {}   ## "correct/wrong" for each question: subject, ID -> correct/wrong, question type
for line in ans_file:
    cols = line.split()
    if len(cols) != 4:
        sys.stderr.write('Answer file error: Wrong format for answer file (maybe a wrong file?): %s' % line)
        sys.exit(1)
    (subject, question_id, type, ans) = cols
    eval_results.setdefault(subject, {})
    answers = dict([x.split('=') for x in ans.split(',')])
    pairid_list = answers.keys()
    if len(pairid_list) < 2:
        sys.stderr.write('Answer file error: Wrong format for answer file at Question ID %s.  Contact the organizer.\n' % question_id)
        sys.exit(1)
    pairid_list.sort(lambda x, y: cmp(int(x), int(y)))
    outputs = ','.join(['%s=%s(%f)' % (id, run_results[id]['label'], run_results[id]['confidence']) for id in pairid_list])
    eval_results[subject][question_id] = { 'type': type, 'answers': ans, 'outputs': outputs }
    if type == 'choose-Y':
        ## question type: choose true statement
        ## output ID with 'Y' with highest confidence
        answer_id = findAnswerID(answers, 'Y', question_id)
        output_id = chooseBestID(pairid_list, 'Y')
        eval_results[subject][question_id]['correct'] = output_id == answer_id
        eval_results[subject][question_id]['choice'] = '%s/%s' % (answer_id, output_id)
    elif type == 'choose-N':
        ## question type: choose false statement
        ## output ID with 'N' with highest confidence
        answer_id = findAnswerID(answers, 'N', question_id)
        output_id = chooseBestID(pairid_list, 'N')
        eval_results[subject][question_id]['correct'] = output_id == answer_id
        eval_results[subject][question_id]['choice'] = '%s/%s' % (answer_id, output_id)
    elif type == 'combination':
        ## question type: choose combinations of true/false
        ## outputs for all t2 must be correct
        wrong_choices = [id for id in pairid_list if answers[id] != run_results[id]['label']]
        eval_results[subject][question_id]['correct'] = len(wrong_choices) == 0
    else:
        sys.stderr.write('Answer file error: Wrong format for answer file at Question ID %s.  Contact the organizer.\n' % question_id)
        sys.exit(1)

ans_file.close()

######################################################################
## show results

subjects = eval_results.keys()
subjects.sort()

sys.stdout.write('===== SUMMARY =====\n')
sum_score = 0
sum_num = 0
for subject in subjects:
    results = eval_results[subject]
    score = len([id for id in results.keys() if results[id]['correct']])
    num = len(results.keys())
    ratio = float(score) / num
    sys.stdout.write('%6s: %f (%d/%d)\n' % (subject, ratio, score, num))
    sum_score = sum_score + score
    sum_num = sum_num + num
sys.stdout.write('%6s: %f (%d/%d)\n' % ('TOTAL', float(sum_score) / sum_num, sum_score, sum_num))

sys.stdout.write('===== EVALUATION RESULTS =====\n')
for subject in subjects:
    sys.stdout.write('%s\n' % subject)
    results = eval_results[subject]
    ids = results.keys()
    ids.sort(lambda x, y: cmp(int(x), int(y)))
    sys.stdout.write('  %2s: %-7s  GLD/SYS    %-12s %-24s %s\n' % ('ID', 'EVAL', 'TYPE', 'GOLD LABELS', 'SYSTEM LABELS'))
    for id in ids:
        judge = 'CORRECT' if results[id]['correct'] else 'WRONG'
        type = results[id]['type']
        answers = results[id]['answers']
        outputs = results[id]['outputs']
        choice = results[id].get('choice', '')
        sys.stdout.write('  %2s: %-7s  %-7s    %-12s %-24s %s\n' % (id, judge, choice, type, answers, outputs))

